﻿using GrapeCity.ActiveReports;
using GrapeCity.ActiveReports.Aspnetcore.Viewer;
using GrapeCity.ActiveReports.Document;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using System;
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;

namespace ReportLoadCancel
{
    public class Startup
    {
        private static readonly string CurrentDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase)?.Replace("file:\\", "");
        public static readonly DirectoryInfo ReportsDirectory = new DirectoryInfo(Path.Combine(CurrentDir, "Reports"));

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);

            services
                .AddLogging(config =>
                {
                    // Disable the default logging configuration
                    config.ClearProviders();

                    // Enable logging for debug mode only
                    if (Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT") == Environments.Development)
                    {
                        config.AddConsole();
                    }
                })
                .AddReporting()
                .AddMvc(option => option.EnableEndpointRouting = false);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseReporting(settings =>
            {
                settings.UseCustomStore(GetReport);
                settings.UseCompression = true;
            });

            app.UseMvc();
        }

        private object GetReport(string arg)
        {
            string documentPath = Path.Combine(ReportsDirectory.FullName, Path.GetFileNameWithoutExtension(arg) + ".rdf");
            if (ReportsDirectory.GetFiles(Path.GetFileName(documentPath)).Length > 0)
            {
                var document = new SectionDocument();
                document.Load(documentPath);
                return document;
            }
            else
            {
                if (Path.GetExtension(arg) == ".rpx")
                {
                    var report = new SectionReport();
                    report.LoadLayout(XmlReader.Create(Path.Combine(ReportsDirectory.FullName, arg)));
                    report.Run();
                    report.Document.Save(documentPath);
                    return report.Document;
                }
                else
                {
                    var report = new PageReport();
                    report.Load(new FileInfo(Path.Combine(ReportsDirectory.FullName, arg)));
                    return report;
                }
            }
        }
    }
}
